@@ -44,8 +44,8 @@ def get_line(start: int, line_ends: List) -> str:
 
 def lint_file(config: Config, this_file: Path) -> Dict:
     """Check file for formatting errors."""
-    file_name = str(this_file)
-    errors: dict = {file_name: []}
+    filename = str(this_file)
+    errors: dict = {filename: []}
     html = this_file.read_text(encoding="utf8")
 
     # build list of line ends for file
@@ -59,9 +59,9 @@ def lint_file(config: Config, this_file: Path) -> Dict:
 
         for pattern in rule["patterns"]:
 
-            # rule H025 is a special case where the output must be an even num.
+            # rule H025 is a special case where the output must be an even number.
             if rule["name"] == "H025":
-                open_tags = []
+                open_tags: List[re.Match] = []
 
                 for match in re.finditer(
                     re.compile(
@@ -69,30 +69,27 @@ def lint_file(config: Config, this_file: Path) -> Dict:
                     ),
                     html,
                 ):
-                    if match.group(1) and not re.match(
+                    if match.group(2) and not re.search(
                         re.compile(
-                            fr"^/?{config.always_self_closing_html_tags}", re.I | re.X
+                            rf"^/?{config.always_self_closing_html_tags}\b", re.I | re.X
                         ),
-                        re.split(r"\s+", match.group(1))[0],
+                        match.group(2),
                     ):
                         # close tags should equal open tags
-                        if re.split(r"\s+", match.group(1))[0][0] != "/":
-                            open_tags.append(match)
+                        if match.group(2)[0] != "/":
+                            open_tags.insert(0, match)
                         else:
                             for i, tag in enumerate(copy.deepcopy(open_tags)):
-                                if (
-                                    re.split(r"\s+", tag.group(1))[0]
-                                    == re.split(r"\s+", match.group(1))[0][1:]
-                                ):
+                                if tag.group(3) == match.group(2)[1:]:
                                     open_tags.pop(i)
                                     break
                             else:
                                 # there was no open tag matching the close tag
-                                open_tags.append(match)
+                                open_tags.insert(0, match)
 
                 for match in open_tags:
                     if inside_ignored_block(config, html, match) is False:
-                        errors[file_name].append(
+                        errors[filename].append(
                             {
                                 "code": rule["name"],
                                 "line": get_line(match.start(), line_ends),
@@ -108,9 +105,8 @@ def lint_file(config: Config, this_file: Path) -> Dict:
                     ),
                     html,
                 ):
-
                     if inside_ignored_block(config, html, match) is False:
-                        errors[file_name].append(
+                        errors[filename].append(
                             {
                                 "code": rule["name"],
                                 "line": get_line(match.start(), line_ends),
@@ -120,10 +116,10 @@ def lint_file(config: Config, this_file: Path) -> Dict:
                         )
 
     # remove duplicate matches
-    for file_name, error_dict in errors.items():
+    for filename, error_dict in errors.items():
         unique_errors = []
         for dict_ in error_dict:
             if dict_ not in unique_errors:
                 unique_errors.append(dict_)
-        errors[file_name] = unique_errors
+        errors[filename] = unique_errors
     return errors
