@@ -6,23 +6,25 @@ Much code is borrowed from https://github.com/rareyman/HTMLBeautify, many thanks
 import difflib
 from pathlib import Path
 
-from .formatter.compress_html import compress_html
-from .formatter.expand_html import expand_html
-from .formatter.indent_html import indent_html
+from .formatter.compress import compress_html
+from .formatter.condense import condense_html
+from .formatter.expand import expand_html
+from .formatter.indent import indent_html
 from .settings import Config
 
 
-def reformat_file(config: Config, check: bool, this_file: Path) -> dict:
+def reformat_file(config: Config, this_file: Path) -> dict:
     """Reformat html file."""
     rawcode = this_file.read_text(encoding="utf8")
 
-    expanded = expand_html(rawcode, config)
-    compressed = compress_html(expanded, config)
-    indented = indent_html(compressed, config)
+    compressed = compress_html(rawcode, config)
+    expanded = expand_html(compressed, config)
+    condensed = condense_html(expanded, config)
+    indented = indent_html(condensed, config)
 
     beautified_code = indented
 
-    if check is not True:
+    if config.check is not True:
         # update the file
         this_file.write_text(beautified_code, encoding="utf8")
 
