@@ -3,60 +3,121 @@
 # flake8: noqa
 
 
+import json
 import logging
-import re
 
 ## get pyproject.toml settings
 from pathlib import Path
 from typing import Dict, List, Optional, Union
 
-import tomlkit
 import yaml
 from click import echo
 from colorama import Fore
+from HtmlTagNames import html_tag_names
+from HtmlVoidElements import html_void_elements
+from pathspec import PathSpec
+from pathspec.patterns.gitwildmatch import GitWildMatchPatternError
+
+try:
+    import tomllib
+except ImportError:
+    import tomli as tomllib  # type: ignore
 
 logger = logging.getLogger(__name__)
 
 
-def find_pyproject(src: Path) -> Optional[Path]:
-    """Search upstream for a pyprojec.toml file."""
-
+def find_project_root(src: Path) -> Path:
+    """Attempt to get the project root."""
     for directory in [src, *src.resolve().parents]:
 
-        candidate = directory / "pyproject.toml"
+        if (directory / ".git").exists():
+            return directory
 
-        if candidate.is_file():
-            return candidate
+        if (directory / ".hg").is_dir():
+            return directory
+
+        if (directory / "pyproject.toml").is_file():
+            return directory
+
+        if (directory / ".djlintrc").is_file():
+            return directory
+
+    # pylint: disable=W0631
+    return directory
+
+
+def load_gitignore(root: Path) -> PathSpec:
+    """Search upstream for a .gitignore file."""
+
+    gitignore = root / ".gitignore"
+    git_lines: List[str] = []
+    if gitignore.is_file():
+        with gitignore.open(encoding="utf-8") as this_file:
+            git_lines = this_file.readlines()
+
+    try:
+        return PathSpec.from_lines("gitwildmatch", git_lines)
+
+    except GitWildMatchPatternError as e:
+        echo(f"Could not parse {gitignore}: {e}", err=True)
+        raise
+
+
+def find_pyproject(root: Path) -> Optional[Path]:
+    """Search upstream for a pyproject.toml file."""
+
+    pyproject = root / "pyproject.toml"
+
+    if pyproject.is_file():
+        return pyproject
 
     return None
 
 
-def find_djlint_rules(src: Path) -> Optional[Path]:
-    """Search upstream for a pyprojec.toml file."""
+def find_djlintrc(root: Path) -> Optional[Path]:
+    """Search upstream for a pyproject.toml file."""
 
-    for directory in [src, *src.resolve().parents]:
+    djlintrc = root / ".djlintrc"
 
-        candidate = directory / ".djlint_rules.yaml"
-
-        if candidate.is_file():
-            return candidate
+    if djlintrc.is_file():
+        return djlintrc
 
     return None
 
 
-def load_pyproject_settings(src: Path) -> Dict:
+def find_djlint_rules(root: Path) -> Optional[Path]:
+    """Search upstream for a pyprojec.toml file."""
+
+    rules = root / ".djlint_rules.yaml"
+
+    if rules.is_file():
+        return rules
+
+    return None
+
+
+def load_project_settings(src: Path) -> Dict:
     """Load djlint config from pyproject.toml."""
 
     djlint_content: Dict = {}
     pyproject_file = find_pyproject(src)
 
     if pyproject_file:
-        content = tomlkit.parse(pyproject_file.read_text(encoding="utf8"))
+        content = tomllib.load(pyproject_file.open("rb"))
         try:
-            djlint_content = content["tool"]["djlint"]  # type: ignore
+            return content["tool"]["djlint"]  # type: ignore
         except KeyError:
             logger.info("No pyproject.toml found.")
 
+    djlintrc_file = find_djlintrc(src)
+
+    if djlintrc_file:
+        try:
+            return json.loads(djlintrc_file.read_text(encoding="utf8"))
+        # pylint: disable=broad-except
+        except BaseException:
+            logger.info("Failed to load .djlintrc file.")
+
     return djlint_content
 
 
@@ -106,8 +167,15 @@ def build_custom_blocks(custom_blocks: Union[str, None]) -> Optional[str]:
     return None
 
 
+def build_custom_html(custom_html: Union[str, None]) -> Optional[str]:
+    """Build regex string for custom HTML blocks."""
+    if custom_html:
+        return "|" + "|".join(x.strip() for x in custom_html.split(","))
+    return None
+
+
 class Config:
-    """Djling Config."""
+    """Djlint Config."""
 
     def __init__(
         self,
@@ -115,62 +183,112 @@ class Config:
         ignore: Optional[str] = None,
         extension: Optional[str] = None,
         indent: Optional[int] = None,
-        quiet: Optional[bool] = False,
+        quiet: bool = False,
         profile: Optional[str] = None,
+        require_pragma: bool = False,
+        reformat: bool = False,
+        check: bool = False,
+        lint: bool = False,
+        use_gitignore: bool = False,
+        warn: bool = False,
     ):
 
-        djlint_settings = load_pyproject_settings(Path(src))
+        self.reformat = reformat
+        self.check = check
+        self.lint = lint
+        self.warn = warn
+        self.stdin = "-" in src
 
+        self.project_root = find_project_root(Path(src))
+
+        djlint_settings = load_project_settings(self.project_root)
+
+        self.gitignore = load_gitignore(self.project_root)
         # custom configuration options
+
+        self.use_gitignore: bool = use_gitignore or djlint_settings.get(
+            "use_gitignore", False
+        )
         self.extension: str = str(extension or djlint_settings.get("extension", "html"))
-        self.quiet: str = str(quiet or djlint_settings.get("quiet", ""))
+        self.quiet: bool = quiet or djlint_settings.get("quiet", False)
+        self.require_pragma: bool = (
+            require_pragma
+            or str(djlint_settings.get("require_pragma", "false")).lower() == "true"
+        )
+
         self.custom_blocks: str = str(
             build_custom_blocks(djlint_settings.get("custom_blocks")) or ""
         )
 
+        self.custom_html: str = str(
+            build_custom_html(djlint_settings.get("custom_html")) or ""
+        )
+
+        self.format_attribute_template_tags: bool = djlint_settings.get(
+            "format_attribute_template_tags", False
+        )
+
         # ignore is based on input and also profile
         self.ignore: str = str(ignore or djlint_settings.get("ignore", ""))
 
         # codes to exclude
         profile_dict: Dict[str, List[str]] = {
+            "html": ["D", "J", "T", "N", "M"],
             "django": ["J", "N", "M"],
             "jinja": ["D", "N", "M"],
             "nunjucks": ["D", "J", "M"],
             "handlebars": ["D", "J", "N"],
             "golang": ["D", "J", "N", "M"],
+            "angular": ["D", "J", "H012", "H026", "H028"],
         }
 
         self.profile_code: List[str] = profile_dict.get(
-            str(profile or djlint_settings.get("profile", "all")).lower(), []
+            str(profile or djlint_settings.get("profile", "html")).lower(), []
         )
         self.profile: str = str(
             profile or djlint_settings.get("profile", "all")
         ).lower()
 
+        self.linter_output_format: str = djlint_settings.get(
+            "linter_output_format", "{code} {line} {message} {match}"
+        )
+
         # load linter rules
         rule_set = validate_rules(
             yaml.load(
                 (Path(__file__).parent / "rules.yaml").read_text(encoding="utf8"),
                 Loader=yaml.SafeLoader,
             )
-            + load_custom_rules(Path(src))
+            + load_custom_rules(self.project_root)
         )
 
         self.linter_rules = list(
             filter(
                 lambda x: x["rule"]["name"] not in self.ignore.split(",")
-                and x["rule"]["name"][0] not in self.profile_code
+                and not any(
+                    x["rule"]["name"].startswith(code) for code in self.profile_code
+                )
                 and self.profile not in x["rule"].get("exclude", []),
                 rule_set,
             )
         )
 
         # base options
-        self.indent: str = (indent or int(djlint_settings.get("indent", 4))) * " "
+        default_indent = 4
+        if not indent:
+            try:
+                indent = int(djlint_settings.get("indent", default_indent))
+            except ValueError:
+                echo(
+                    Fore.RED
+                    + f"Error: Invalid pyproject.toml indent value {djlint_settings['indent']}"
+                )
+                indent = default_indent
+        self.indent: str = indent * " "
 
         default_exclude: str = r"""
             \.venv
-            | venv
+            | venv/
             | \.tox
             | \.eggs
             | \.git
@@ -179,13 +297,13 @@ class Config:
             | \.nox
             | \.svn
             | \.bzr
-            | _build
-            | buck-out
-            | build
-            | dist
+            | _build/
+            | buck-out/
+            | build/
+            | dist/
             | \.pants\.d
             | \.direnv
-            | node_modules
+            | node_modules/
             | __pypackages__
         """
 
@@ -195,7 +313,7 @@ class Config:
 
         if extend_exclude:
             self.exclude += r" | " + r" | ".join(
-                re.escape(x.strip()) for x in extend_exclude.split(",")
+                x.strip() for x in extend_exclude.split(",")
             )
 
         # add blank line after load tags
@@ -213,8 +331,11 @@ class Config:
             | [^\{]{\#
             | <pre
             | <textarea
-            | {%[ ]djlint:off[ ]%}
+            | {%[ ]*?blocktrans(?:late)?[^(?:%})]*?%}
+            | {\#\s*djlint\:\s*off\s*\#}
             | {%[ ]+?comment[ ]+?[^(?:%})]*?%}
+            | {{!--\s*djlint\:off\s*--}}
+            | {{-?\s*/\*\s*djlint\:off\s*\*/\s*-?}}
         """
 
         self.ignored_block_closing: str = r"""
@@ -223,144 +344,35 @@ class Config:
             | \?>
             | </script
             |  -->
-            | \#}
+            # | \#}
             | </pre
             | </textarea
-            | {%[ ]djlint:on[ ]%}
+            | {\#\s*djlint\:\s*on\s*\#}
             | {%[ ]+?endcomment[ ]+?%}
+            | {{!--\s*djlint\:on\s*--}}
+            | {{-?\s*/\*\s*djlint\:on\s*\*/\s*-?}}
+            | {%[ ]*?endblocktrans(?:late)?[^(?:%})]*?%}
+        """
+
+        # ignored block closing tags that
+        # we can safely indent.
+        self.safe_closing_tag: str = r"""
+              </script
+            | </style
+            | {\#\s*djlint\:\s*on\s*\#}
+            | {%[ ]+?endcomment[ ]+?%}
+            | {{!--\s*djlint\:on\s*--}}
+            | {{-?\s*/\*\s*djlint\:on\s*\*/\s*-?}}
         """
 
         # all html tags possible
-        self.indent_html_tags: str = r"""
-              a
-            | abbr
-            | acronym
-            | address
-            | applet
-            | area
-            | article
-            | aside
-            | audio
-            | b
-            | base
-            | basefont
-            | bdi
-            | bdo
-            | big
-            | blockquote
-            | body
-            | br
-            | button
-            | canvas
-            | caption
-            | center
-            | cite
-            | code
-            | col
-            | colgroup
-            | data
-            | datalist
-            | dd
-            | del
-            | details
-            | dfn
-            | dialog
-            | dir
-            | div
-            | dl
-            | dt
-            | em
-            | embed
-            | fieldset
-            | figcaption
-            | figure
-            | font
-            | footer
-            | form
-            | frame
-            | frameset
-            | h1
-            | h2
-            | h3
-            | h4
-            | h5
-            | h6
-            | head
-            | header
-            | hr
-            | html
-            | i
-            | iframe
-            | icon
-            | img
-            | input
-            | ins
-            | kbd
-            | label
-            | legend
-            | li
-            | link
-            | main
-            | map
-            | mark
-            | meta
-            | meter
-            | nav
-            | noframes
-            | noscript
-            | object
-            | ol
-            | optgroup
-            | option
-            | output
-            | p
-            | path
-            | param
-            | picture
-            | progress
-            | q
-            | rp
-            | rt
-            | ruby
-            | s
-            | samp
-            | script
-            | section
-            | select
-            | small
-            | source
-            | span
-            | strike
-            | strong
-            | style
-            | sub
-            | summary
-            | sup
-            | svg
-            | table
-            | tbody
-            | td
-            | template
-            | tfoot
-            | th
-            | thead
-            | time
-            | title
-            | tr
-            | track
-            | tt
-            | u
-            | ul
-            | var
-            | video
-            | wbr
-        """
+        self.indent_html_tags: str = "|".join(html_tag_names) + self.custom_html
 
         self.indent_template_tags: str = (
             r"""  if
                 | for
-                | block
-                | else
+                | block(?!trans|translate)
+            #    | blocktrans(?:late)?[ ]+?trimmed
                 | spaceless
                 | compress
                 | addto
@@ -371,43 +383,27 @@ class Config:
                 | autoescape
                 | filter
                 | each
+                | macro
+                | raw
             """
             + self.custom_blocks
         )
 
-        # the contents of these tag blocks will be indented, then unindented
-        self.tag_indent: str = (
+        self.template_indent: str = (
             r"""
-              (?:\{\{\#|\{%-?)[ ]*?
+            (?:\{\{\#|\{%-?)[ ]*?
                 ("""
             + self.indent_template_tags
             + r"""
-                )
-            | (?:<
-                (?:
-                    """
-            + self.indent_html_tags
-            + """
-                )
-              )
-        """
+            )"""
         )
 
-        self.tag_unindent: str = (
-            r"""^
-              (?:
-                  (?:\{\{\/)
-                | (?:\{%-?[ ]*?end)
-              )
-            | (?:</
+        self.template_unindent: str = r"""
                 (?:
-                    """
-            + self.indent_html_tags
-            + """
-                )
+                  (?:\{\{\/)
+                | (?:\{%-?[ ]*?end(?!comment))
               )
-        """
-        )
+            """
 
         # these tags should be unindented and next line will be indented
         self.tag_unindent_line: str = r"""
@@ -426,7 +422,27 @@ class Config:
         # if lines are longer than x
         self.max_line_length = 120
 
-        self.format_long_attributes = True
+        try:
+            self.max_line_length = int(
+                djlint_settings.get("max_line_length", self.max_line_length)
+            )
+        except ValueError:
+            echo(
+                Fore.RED
+                + f"Error: Invalid pyproject.toml max_line_length value {djlint_settings['max_line_length']}"
+            )
+
+        self.max_attribute_length = 70
+
+        try:
+            self.max_attribute_length = int(
+                djlint_settings.get("max_attribute_length", self.max_attribute_length)
+            )
+        except ValueError:
+            echo(
+                Fore.RED
+                + f"Error: Invalid pyproject.toml max_attribute_length value {djlint_settings['max_attribute_length']}"
+            )
 
         # pattern used to find attributes in a tag
         # order is important.
@@ -456,19 +472,20 @@ class Config:
             | (?:[^\s]+?=(?:\".*?\"|\'.*?\'))
             | required
             | checked
-            | [\w|-]+
-            | [\w|-]+=[\w|-]+
+            | (?:\w|-|\.)+
+            | (?:\w|-|\.)+=(?:\w|-)+
             | {{.*?}}
+            | {%.*?%}
         """
         )
 
-        self.attribute_style_pattern: str = r"(.*?)(style=)([\"|'])(([^\"']+?;)+?)\3"
+        self.attribute_style_pattern: str = r"^(.*?)(style=)([\"|'])(([^\"']+?;)+?)\3"
 
         self.start_template_tags: str = (
             r"""
               if
             | for
-            | block
+            | block(?!trans)
             | spaceless
             | compress
             | load
@@ -481,6 +498,8 @@ class Config:
             | filter
             | verbatim
             | each
+            | macro
+            | raw
             """
             + self.custom_blocks
             + r"""
@@ -490,26 +509,37 @@ class Config:
         self.break_template_tags: str = (
             r"""
               if
-            | end
+            | endif
             | for
-            | block
-            | endblock
+            | endfor
+            | block(?!trans)
+            | endblock(?!trans)
             | else
             | spaceless
+            | endspaceless
             | compress
+            | endcompress
             | load
             | include
             | assets
+            | endassets
             | addto
             | language
             | with
-            | assets
+            | endwith
             | autoescape
+            | endautoescape
             | filter
+            | endfilter
             | elif
             | resetcycle
             | verbatim
+            | endverbatim
             | each
+            | macro
+            | endmacro
+            | raw
+            | endraw
             """
             + self.custom_blocks
             + r"""
@@ -517,22 +547,34 @@ class Config:
         )
 
         self.ignored_blocks: str = r"""
-              <(script|style|pre|textarea).*?</(\1)>
-            | {%[ ]djlint:off[ ]%}.*?{%[ ]djlint:on[ ]%}
+              <(pre|textarea).*?</(\1)>
+            | <(script|style).*?(?=(\</(?:\3)>))
+            # html comment
+            | <!--\s*djlint\:off\s*-->.*?(?=<!--\s*djlint\:on\s*-->)
+            # django/jinja/nunjucks
+            | {\#\s*djlint\:\s*off\s*\#}.*?(?={\#\s*djlint\:\s*on\s*\#})
+            | {%\s*comment\s*%\}\s*djlint\:off\s*\{%\s*endcomment\s*%\}.*?(?={%\s*comment\s*%\}\s*djlint\:on\s*\{%\s*endcomment\s*%\})
+            # handlebars
+            | {{!--\s*djlint\:off\s*--}}.*?(?={{!--\s*djlint\:on\s*--}})
+            # golang
+            | {{-?\s*/\*\s*djlint\:off\s*\*/\s*-?}}.*?(?={{-?\s*/\*\s*djlint\:on\s*\*/\s*-?}})
             | <!--.*?-->
-            | {\*.*?\*}
-            | {\#.*?\#}
             | <\?php.*?\?>
-            | {\%[ ]trans[ ][^}]*?\%}
-            | {%[ ]+?comment[ ]+?[^(?:%})]*?%}.*?{%[ ]+?endcomment[ ]+?%}
+            | {%[ ]*?blocktranslate\b[^(?:%})]*?%}.*?{%[ ]*?endblocktranslate[ ]*?%}
+            | {%[ ]*?blocktrans\b[^(?:%})]*?%}.*?{%[ ]*?endblocktrans[ ]*?%}
+            | {%[ ]*?comment\b[^(?:%})]*?%}.*?(?={%[ ]*?endcomment[ ]*?%})
+            | ^---[\s\S]+?---
         """
 
         self.ignored_inline_blocks: str = r"""
               <!--.*?-->
+            | <(script|style).*?\</(?:\1)>
             | {\*.*?\*}
-            | {\#.*?\#}
+            | {\#(?!.*djlint:[ ]*?(?:off|on)\b).*\#}
             | <\?php.*?\?>
-            | {%[ ]+?comment[ ]+?[^(?:%})]*?%}.*?{%[ ]+?endcomment[ ]+?%}
+            | {%[ ]*?comment\b[^(?:%})]*?%}.*?{%[ ]*?endcomment[ ]*?%}
+            | {%[ ]*?blocktranslate\b[^(?:%})]*?%}.*?{%[ ]*?endblocktranslate[ ]*?%}
+            | {%[ ]*?blocktrans\b[^(?:%})]*?%}.*?{%[ ]*?endblocktrans[ ]*?%}
         """
 
         self.optional_single_line_html_tags: str = r"""
@@ -559,16 +601,12 @@ class Config:
             | li
             | script
             | style
+            | head
+            | body
+            | p
         """
 
-        self.always_self_closing_html_tags: str = r"""
-              link
-            | img
-            | meta
-            | source
-            | br
-            | input
-        """
+        self.always_self_closing_html_tags: str = "|".join(html_void_elements)
 
         self.optional_single_line_template_tags: str = r"""
               if
@@ -577,12 +615,13 @@ class Config:
             | with
         """
 
-        self.break_html_tags: str = r"""
+        self.break_html_tags: str = (
+            r"""
               html
             | head
             | body
             | div
-            | a
+         #   | a # a gets no breaks #177
             | nav
             | ul
             | ol
@@ -613,17 +652,51 @@ class Config:
             | figure
             | figcaption
             | video
-            | span
+         #   | span # span gets no breaks #171
             | p
             | g
             | svg
             | h\d
             | button
-            | img
             | path
             | picture
             | script
             | style
             | details
             | summary
+            | """
+            + self.always_self_closing_html_tags
+            + self.custom_html
+            + """
         """
+        )
+
+        # the contents of these tag blocks will be indented, then unindented
+        self.tag_indent: str = (
+            self.template_indent
+            + """
+            | (?:<
+                (?:
+                    """
+            + self.indent_html_tags
+            + """
+                )\\b
+              )
+        """
+        )
+
+        self.tag_unindent: str = (
+            r"""
+                ^
+                """
+            + self.template_unindent
+            + """
+            | (?:</
+                (?:
+                    """
+            + self.indent_html_tags
+            + """
+                )\\b
+              )
+        """
+        )
